#property      copyright "Copyright 2018, Cowen Funding Services, LLC"
#property      link      "http://www.cowen.tv"
#property      version   "1.00"
#include       <Expert\Expert.mqh>
#include       <Expert\Trailing\TrailingParabolicSAR.mqh>
#include       <Trade\PositionInfo.mqh>
#include       <Trade\Trade.mqh>                                        // trade position object
CTrade         trade;                                                   // trading object
CTrade         myTradingControlPanel;
//---          available money management                                //
#include       <Expert\Money\MoneyFixedLot.mqh>
//---          input parameters                                          //
input int      StopLoss=3000;                // Stop Loss
input int      TakeProfit=4000;              // Take Profit
input int      ADX_Period=19;                // ADX Period
input int      MA_Period=9;                  // Moving Average Period
input int      EA_Magic=12345;               // EA Magic Number
input double   Adx_Min=22.0;                 // Minimum ADX Value
input double   Lot=1;                        // Lots to Trade
//---          Other parameters
int            adxHandle; // handle for our ADX indicator
int            maHandle;  // handle for our Moving Average indicator
double         plsDI[],minDI[],adxVal[]; // Dynamic arrays to hold the values of +DI, -DI and ADX values for each bars
double         maVal[]; // Dynamic array to hold the values of Moving Average for each bars
double         p_close; // Variable to store the close value of a bar
int            STP, TKP;   // To be used for Stop Loss & Take Profit values
//------------------------------------------------------------------+
//             Expert initialization function                                   |
//------------------------------------------------------------------+
int OnInit()
  {
//---          Get handle for ADX indicator
               adxHandle=iADX(NULL,0,ADX_Period);
//---          Get the handle for Moving Average indicator
               maHandle=iMA(_Symbol,_Period,MA_Period,0,MODE_EMA,PRICE_CLOSE);
//---          What if handle returns Invalid Handle
               if(adxHandle<0 || maHandle<0)
                  {
                     Alert("Error Creating Handles for indicators - error: ",GetLastError(),"!!");
                     return(-1);
                  }

//--- Let us handle currency pairs with 5 or 3 digit prices instead of 4
   STP = StopLoss;
   TKP = TakeProfit;
   if(_Digits==5 || _Digits==3)
     {
      STP = STP*10;
      TKP = TKP*10;
     }
   return(0);
  }
//------------------------------------------------------------------+
// Expert deinitialization function                                 |
//------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- Release our indicator handles
   IndicatorRelease(adxHandle);
   IndicatorRelease(maHandle);
  }
//------------------------------------------------------------------+
// Expert tick function                                             |
//------------------------------------------------------------------+
void OnTick()
{
//--- Do we have enough bars to work with
   if(Bars(_Symbol,_Period)<60) // if total bars is less than 60 bars
     {
      Alert("We have less than 60 bars, EA will now exit!!");
      return;
     }  

// We will use the static Old_Time variable to serve the bar time.
// At each OnTick execution we will check the current bar time with the saved one.
// If the bar time isn't equal to the saved time, it indicates that we have a new tick.

   static datetime Old_Time;
   datetime New_Time[1];
   bool IsNewBar=false;

// copying the last bar time to the element New_Time[0]
   int copied=CopyTime(_Symbol,_Period,0,1,New_Time);
   if(copied>0) // ok, the data has been copied successfully
     {
      if(Old_Time!=New_Time[0]) // if old time isn't equal to new bar time
        {
         IsNewBar=true;   // if it isn't a first call, the new bar has appeared
         if(MQL5InfoInteger(MQL5_DEBUGGING)) Print("We have new bar here ",New_Time[0]," old time was ",Old_Time);
         Old_Time=New_Time[0];            // saving bar time
        }
     }
   else
     {
      Alert("Error in copying historical times data, error =",GetLastError());
      ResetLastError();
      return;
     }

//--- EA should only check for new trade if we have a new bar
   if(IsNewBar==false)
     {
      return;
     }
 
//--- Do we have enough bars to work with
   int Mybars=Bars(_Symbol,_Period);
   if(Mybars<60) // if total bars is less than 60 bars
     {
      Alert("We have less than 60 bars, EA will now exit!!");
      return;
     }

//--- Define some MQL5 Structures we will use for our trade
   MqlTick latest_price;      // To be used for getting recent/latest price quotes
   MqlTradeRequest mrequest;  // To be used for sending our trade requests
   MqlTradeResult mresult;    // To be used to get our trade results
   MqlRates mrate[];          // To be used to store the prices, volumes and spread of each bar
   ZeroMemory(mrequest);      // Initialization of mrequest structure
/*
     Let's make sure our arrays values for the Rates, ADX Values and MA values 
     is store serially similar to the timeseries array
*/
// the rates arrays
   ArraySetAsSeries(mrate,true);
// the ADX DI+values array
   ArraySetAsSeries(plsDI,true);
// the ADX DI-values array
   ArraySetAsSeries(minDI,true);
// the ADX values arrays
   ArraySetAsSeries(adxVal,true);
// the MA-8 values arrays
   ArraySetAsSeries(maVal,true);

//--- Get the last price quote using the MQL5 MqlTick Structure
   if(!SymbolInfoTick(_Symbol,latest_price))
     {
      Alert("Error getting the latest price quote - error:",GetLastError(),"!!");
      return;
     }

//--- Get the details of the latest 3 bars
   if(CopyRates(_Symbol,_Period,0,3,mrate)<0)
     {
      Alert("Error copying rates/history data - error:",GetLastError(),"!!");
      ResetLastError();
      return;
     }

//--- Copy the new values of our indicators to buffers (arrays) using the handle
   if(CopyBuffer(adxHandle,0,0,3,adxVal)<0 || CopyBuffer(adxHandle,1,0,3,plsDI)<0
      || CopyBuffer(adxHandle,2,0,3,minDI)<0)
     {
      Alert("Error copying ADX indicator Buffers - error:",GetLastError(),"!!");
      ResetLastError();
      return;
     }
   if(CopyBuffer(maHandle,0,0,3,maVal)<0)
     {
      Alert("Error copying Moving Average indicator buffer - error:",GetLastError());
      ResetLastError();
      return;
     }
//--- we have no errors, so continue
//--- Declare bool type variables and conditions to buy, sell or hold
   bool Buy_opened=false;  // variable to hold the result of Buy opened position
   bool Sell_opened=false; // variables to hold the result of Sell opened position
//-------------------------------------------------------------------------------------------------------------+

//          C R O S S I N G    A V E R A G E S            

//-------------------------------------------------------------------------------------------------------------+

// adxVal   ----------------------------------------------------------------------------------------------------+

            bool  adxVal_x_minDI                    // adxVal "Orange" crossing down over DI(-) "Red"
            =     (adxVal[1] > minDI[1]) 
            &&    (adxVal[0] < minDI[0]);                       
            
            bool adxVal_x_plsDI                    // adxVal "Orange" crossing down over DI(+) "Green"
            =     (adxVal[1] > plsDI[1]) 
            &&    (adxVal[0] < plsDI[0]);                       
            
            bool adxVal_x_maVal                    // adxVal "Orange" crossing down over maVal "Blue"
            =     (adxVal[1] > maVal[1]) 
            &&    (adxVal[0] < maVal[0]);                       
            
// maVal    ----------------------------------------------------------------------------------------------------+
            
            bool maVal_x_adxVal                    // maVal "Blue" crossing down over adxVal "Orange"
            =     (adxVal[1] < maVal[1]) 
            &&    (adxVal[0] > maVal[0]);                                             
            
            bool maVal_x_minDI                    // maVal "Blue" crossing down over DI(-)minDI "Red"
            =     (minDI[1] < maVal[1]) 
            &&    (minDI[0] > maVal[0]);                       
            
            bool maVal_x_plsDI                    // maVal "Blue" crossing down over DI(+)plsDI "Green"
            =     (plsDI[1] < maVal[1]) 
            &&    (plsDI[0] > maVal[0]);                       
            
// minDI   ----------------------------------------------------------------------------------------------------+
         
            bool minDI_x_adxVal                    // minDI (DI-) "Red" crossing down over adxVal "Orange"
            =     (adxVal[1] < minDI[1]) 
            &&    (adxVal[0] > minDI[0]);                       
            
            bool minDI_x_plsDI                    // DI(-)"Red" crossing down over DI(+) "Green"
            =     (minDI[1]  > plsDI[1]) 
            &&    (minDI[0]  < plsDI[0]);                       
            
            bool minDI_x_maVal                    // minDI "Red" crossing down over maVal "Blue"
            =     (minDI[1]  > maVal[1]) 
            &&    (minDI[0]  < maVal[0]);    
            
// plsDI    ----------------------------------------------------------------------------------------------------+                
            
            bool plsDI_x_adxVal                    // minDI (DI+) "Green" crossing down over adxVal "Orange"
            =     (adxVal[1] < plsDI[1]) 
            &&    (adxVal[0] > plsDI[0]); 
            
            bool plsDI_x_maVal                    // DI(+) plsDI "Green" crossing down over maVal "Blue"
            =     (plsDI[1]  > maVal[1]) 
            &&    (plsDI[0]  < maVal[0]);                                                                          
            
            bool plsDI_x_minDI                    // DI(+)"Green" crossing down over DI(-) "Red"
            =     (minDI[1]  < plsDI[1]) 
            &&    (minDI[0]  > plsDI[0]);                       
                                    
//-------------------------------------------------------------------------------------------------------------+

//          R E L A T I V E     P O S I T I O N S            

//-------------------------------------------------------------------------------------------------------------+ 
            
// adxVal   ----------------------------------------------------------------------------------------------------+

            bool adxVal_above_maVal
            =(adxVal[0]  > maVal[0]);               //  maVal "Blue" above DI(+) "Green" / plsDI

            bool adxVal_below_maVal               //  maVal "Blue" below DI(+) "Green" / plsDI
            =     (adxVal[0] < maVal[0]);
            
            bool adxVal_above_plsDI               // adxVal "Orange" above plsDI "Green"
            =     (adxVal[0] > plsDI[0]);                        
            
            bool adxVal_below_plsDI               // adxVal "Orange" below plsDI "Green"
            =     (adxVal[0] < plsDI[0]);                        
   
            bool adxVal_above_minDI               // adxVal "Orange" above minDI "Red"
            =     (adxVal[0] > minDI[0]);                       
            
            bool adxVal_below_minDI               // adxVal "Orange" below minDI "Red"
            =     (adxVal[0] < minDI[0]);                       
 
 // maVal   ----------------------------------------------------------------------------------------------------+
   
            bool maVal_above_adxVal                //  maVal "Blue" above adxVal "Orange"
            =     (maVal[0] > adxVal[0]);
            
            bool maVal_below_adxVal                //  maVal "Blue" below adxVal "Orange"
            =     (maVal[0] < adxVal[0]);
            
            bool maVal_above_plsDI                //  maVal "Blue" above DI(+) "Green" / plsDI
            =     (maVal[0] > plsDI[0]+3);

            bool maVal_below_plsDI                //  maVal "Blue" below DI(+) "Green" / plsDI
            =     (maVal[0] < plsDI[0]);
            
            bool maVal_above_minDI                //  maVal "Blue" above DI(-) "Red" / minDI
            =     (maVal[0] > minDI[0]);
            
            bool maVal_below_minDI                //  maVal "Blue" below DI(-) "Red" / minDI
            =     (maVal[0] < minDI[0]);

// plsDI   ----------------------------------------------------------------------------------------------------+
   
            bool plsDI_above_adxVal                //  plsDI "Green" above adxVal "Orange"
            =     (plsDI[0] > adxVal[0]);
            
            bool plsDI_below_adxVal                //  plsDI "Green" below adxVal "Orange"
            =     (plsDI[0] < adxVal[0]);
            
            bool plsDI_above_maVal                //  plsDI "Green" above maVal "Blue"
            =     (plsDI[0] > maVal[0]);

            bool plsDI_below_maVal                //  plsDI "Green" below maVal "Blue"
            =     (plsDI[0] < maVal[0]);
            
            bool plsDI_above_minDI                //  plsDI "Green" above DI(-) "Red" / minDI
            =     (plsDI[0] > minDI[0]);
            
            bool plsDI_below_minDI                //  plsDI "Green" below DI(-) "Red" / minDI
            =     (plsDI[0] < minDI[0]);            
            
// minDI   ----------------------------------------------------------------------------------------------------+
   
            bool minDI_above_adxVal                //  minDI "Red" above adxVal "Orange"
            =     (minDI[0] > adxVal[0]);
            
            bool minDI_below_adxVal                //  minDI "Red" below adxVal "Orange"
            =     (minDI[0] < adxVal[0]);
            
            bool minDI_above_plsDI                //  minDI "Red" above DI(+) "Green" / plsDI
            =     (minDI[0] > plsDI[0]);

            bool minDI_below_plsDI                //  minDI "Red" below DI(+) "Green" / plsDI
            =     (minDI[0] < plsDI[0]);
            
            bool minDI_above_maVal                //  minDI "Red" above maVal "Blue"
            =     (minDI[0] > maVal[0]);
            
            bool minDI_below_minDI                //  minDI "Red" below maVal "Blue"
            =     (minDI[0]  < maVal[0]);

//-------------------------------------------------------------------------------------------------------------+

//          S L O P  E S            

//-------------------------------------------------------------------------------------------------------------+ 
            
            bool adxVal_up_slope                   //  slope of adx Value increasing 
            =     (adxVal[2]  < adxVal[1])         
            &     (adxVal[1]  < adxVal[0]);              

            bool adxVal_down_slope                 //  slope of adx Value decreasing
            =     (adxVal[2]  > adxVal[1])          
            &     (adxVal[1]  > adxVal[0]);              

            bool maVal_up_slope                   //  slope of ma Value increasing 
            =     (maVal[2]  < maVal[1])         
            &     (maVal[1]  < maVal[0]);              

            bool maVal_down_slope                 //  slope of ma Value decreasing
            =     (maVal[2]  > maVal[1])          
            &     (maVal[1]  > maVal[0]);  
            
            bool plsDI_up_slope                   //  slope of D+ Value increasing 
            =     (plsDI[2]  < plsDI[1])         
            &     (plsDI[1]  < plsDI[0]);              

            bool plsDI_down_slope                 //  slope of D+ Value decreasing
            =     (plsDI[2]  > plsDI[1])          
            &     (plsDI[1]  > plsDI[0]);             
            
            bool minDI_up_slope                   //  slope of D- Value increasing 
            =     (minDI[2]  < minDI[1])         
            &     (minDI[1]  < minDI[0]);              

            bool minDI_down_slope                 //  slope of D- Value decreasing
            =     (minDI[2]  > minDI[1])          
            &     (minDI[1]  > minDI[0]);         
                
//---------------------------------------------------------------------------------------------------------------
//
//    CHECK FOR OPEN POSITIONS
//
//------------------------------------------------------------------------------------------------------------------
//--- Do we have positions opened already?
         
   if(PositionSelect(_Symbol)==true) // we have an opened position
     {
      if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
        {
         Buy_opened=true;  //It is a Buy
         
         if(         (plsDI_x_adxVal)
            &&       (maVal_above_adxVal)
            &&       (minDI_below_plsDI)
            ||       (adxVal_x_minDI) 
            &&       (plsDI_above_minDI)
            &&       (maVal_above_plsDI)
            ||       (plsDI_x_minDI)
            &&       (adxVal_above_plsDI)
            &&       (maVal_above_adxVal)
            ||       (adxVal_x_plsDI)
            &&       (minDI_above_adxVal)
            &&       (maVal_above_minDI)
            )
            
            {  
            // Getting Errors and not able to move forward - please help (files attached) 
            // Alert: The Sell order request could not be completed -error:4753
            // Alert: The Sell order request could not be completed -error:0

            // Closing All Open Positions
                  { 
                     trade.PositionClose(_Symbol);   //ClosePosition(); 
                  }
            //myTradingControlPanel.SetTypeFilling(ORDER_FILLING_IOC); 
              //    int i=PositionsTotal()-1;
                //  while (i>=0)
                  //{
                    // if (trade.PositionClose(PositionGetSymbol(i))) i--;
                  //}
            
            //{ 
            //trade.PositionClose(_Symbol);
                  Alert("open Buy Position Closed"/*,datetime()*/);
                  return;    // End Loop
            }
        }
         else if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_SELL)
        {
         Sell_opened=true; // It is a Sell
         
            if(      (maVal_x_plsDI)
                  && (adxVal_above_minDI)
                  && (minDI_above_plsDI)
                  || (maVal_x_adxVal)  
                  && (minDI_above_maVal)
                  && (plsDI_above_minDI)
                  || (minDI_x_plsDI)
                  && (adxVal_above_minDI)
                  && (maVal_below_plsDI)
                  || (maVal_x_adxVal)
                  && (minDI_above_maVal)
                  && (plsDI_below_maVal)
                  || (maVal_x_minDI)
                  && (adxVal_below_minDI)
                  && (plsDI_above_maVal)
                  || (minDI_x_plsDI)
                  && (maVal_below_plsDI)
                  && (adxVal_x_maVal)
               )    
               {  // Closing All Open Positions
                  { 
                     trade.PositionClose(_Symbol);   //ClosePosition();
                  }
                  //myTradingControlPanel.SetTypeFilling(ORDER_FILLING_IOC); 
                  //int i=PositionsTotal()-1;
                  //while (i>=0)
                  //{
                  //   if (trade.PositionClose(PositionGetSymbol(i))) i--;
                  //}
               
               //trade.PositionClose(_Symbol);
                  Alert("open Sell Position Closed"/*,datetime()*/);
                  return;    // End Loop
               }
         }
       }
//-------------------------------------------------------------------------------------------------------------+
// Copy the bar close price for the previous bar prior to the current bar, that is Bar 1

   p_close=mrate[1].close;  // bar 1 close price

//-------------------------------------------------------------------------------------------------------------+

//          L O N G   S E T - U P S   F O R   B U Y S

//-------------------------------------------------------------------------------------------------------------+/*
//    1. Check for a long/Buy Setup : MA-8 increasing upwards, 
    //previous price close above it, ADX > 22, +DI > -DI
//
//--- Putting all together   
   
      if(            (maVal_x_plsDI)
                  && (adxVal_above_minDI)
                  && (minDI_above_plsDI)
                  || (maVal_x_adxVal)  
                  && (minDI_above_maVal)
                  && (plsDI_above_minDI)
                  || (minDI_x_plsDI)
                  && (adxVal_above_minDI)
                  && (maVal_below_plsDI)
                  || (maVal_x_adxVal)
                  && (minDI_above_maVal)
                  && (plsDI_below_maVal)
                  || (maVal_x_minDI)
                  && (adxVal_below_minDI)
                  && (plsDI_above_maVal)
                  || (minDI_x_plsDI)
                  && (maVal_below_plsDI)
                  && (adxVal_x_maVal)
         )    
      {  // any opened Buy position?
         if(Buy_opened)
         {
            Alert("Buy Position Already Open"/*,datetime()*/);
            return;    // Don't open a new Buy Position
         }
         ZeroMemory(mrequest);
         mrequest.action = TRADE_ACTION_DEAL;                                  // immediate order execution
         mrequest.price = NormalizeDouble(latest_price.ask,_Digits);           // latest ask price
         mrequest.sl = NormalizeDouble(latest_price.ask - STP*_Point,_Digits); // Stop Loss
         mrequest.tp = NormalizeDouble(latest_price.ask + TKP*_Point,_Digits); // Take Profit
         mrequest.symbol = _Symbol;                                            // currency pair
         mrequest.volume = Lot;                                                 // number of lots to trade
         mrequest.magic = EA_Magic;                                             // Order Magic Number
         mrequest.type = ORDER_TYPE_BUY;                                        // Buy Order
         mrequest.type_filling = ORDER_FILLING_RETURN;                           // Order execution type
         mrequest.deviation=0.5;                                                   // Deviation from current price
         //--- send order
         OrderSend(mrequest,mresult);
         // get the result code
         if(mresult.retcode==10009 || mresult.retcode==10008) //Request is completed or order placed
           {
            Alert("A Buy order has been successfully placed with Ticket#:",mresult.order,"!!");
           }
         else
           {
            Alert("The Buy order request could not be completed -error:",GetLastError());
            ResetLastError();           
            return;
           }
       }    // End Conditional Buy Loop
//----------------------------------------------------------------------------------------+

//            SHORT SELL SETUPS

//----------------------------------------------------------------------------------------+
/*
    2. Check for a Short/Sell Setup : MA-8 decreasing downwards, 
    previous price close below it, ADX > 22, -DI > +DI
*/
//--- Putting all together
      {     
            if
            (        (plsDI_x_adxVal)
            &&       (maVal_above_adxVal)
            &&       (minDI_below_plsDI)
            ||       (adxVal_x_minDI) 
            &&       (plsDI_above_minDI)
            &&       (maVal_above_plsDI)
            ||       (plsDI_x_minDI)
            &&       (adxVal_above_plsDI)
            &&       (maVal_above_adxVal)
            ||       (adxVal_x_plsDI)
            &&       (minDI_above_adxVal)
            &&       (maVal_above_minDI)
             )
         { 
            // any opened Sell position?
            if(Sell_opened)
            {
               Alert("Sell positionAlready Open"/*,datetime()*/);
               return;    // Don't open a new Sell Position
            }
         ZeroMemory(mrequest);
         mrequest.action=TRADE_ACTION_DEAL;                                // immediate order execution
         mrequest.price = NormalizeDouble(latest_price.bid,_Digits);           // latest Bid price
         mrequest.sl = NormalizeDouble(latest_price.bid + STP*_Point,_Digits); // Stop Loss
         mrequest.tp = NormalizeDouble(latest_price.bid - TKP*_Point,_Digits); // Take Profit
         mrequest.symbol = _Symbol;                                          // currency pair
         mrequest.volume = Lot;                                              // number of lots to trade
         mrequest.magic = EA_Magic;                                          // Order Magic Number
         mrequest.type= ORDER_TYPE_SELL;                                     // Sell Order
         mrequest.type_filling = ORDER_FILLING_RETURN;                          // Order execution type
         mrequest.deviation=0.5;                                             // Deviation from current price
         //--- send order
         OrderSend(mrequest,mresult);
         // get the result code
         if(mresult.retcode==10009 || mresult.retcode==10008) //Request is completed or order placed
           {
            Alert("A Sell order has been successfully placed with Ticket#:",mresult.order,"!!");
           }
         }
         else
           {
            Alert("The Sell order request could not be completed -error:",GetLastError());
            ResetLastError();
            return;
           }
        }
        return;
      }

//------------------------------------------------------------------+

//+------------------------------------------------------------------+


//+------------------------------------------------------------------+

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//|                     FUNCTIONS LIBRARY                                   
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/*

Content:
1) ClosePosition

*/

//+------------------------------------------------------------------+
//| CLOSE POSITION                                                
//+------------------------------------------------------------------+
void ClosePosition()
   {
// Type: Fixed Template 
// Do not edit unless you know what you're doing

// This function closes our all current position of the current symbol. Checks for basic errors.

   myTradingControlPanel.PositionClose(_Symbol); // Closes position related to this symbol
   
   if(myTradingControlPanel.ResultRetcode()==10008 || myTradingControlPanel.ResultRetcode()==10009) //Request is completed or order placed
      {
      Print("Exit rules: A close order has been successfully placed with Ticket#: ",myTradingControlPanel.ResultOrder());
      }
   else
      {
      Print("Exit rules: The close order request could not be completed.Error: ",GetLastError());
      ResetLastError();
      }
  }

//+------------------------------------------------------------------+
//| End of CLOSE POSITION                                              
//+------------------------------------------------------------------+